// X3_SD16DeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X3_SD16_Mb.h>
#include <X3_SD16DeviceMap_Mb.h>
#include <X3MemoryMap_Mb.h>
#include <SystemSupport_Mb.h>

using namespace std;

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X3_MemoryMap;

//---------------------------------------------------------
//  Clock and Module Constants Definition
const double  kVcxoPowerOnRate = 10.0f;       // Si570 power on rate (MHz)
const int     VcxoI2C_SubAddr  = 0xAA;        //

//---------------------------------------------------------
//  Clock and Module Constants Definition

const int    InputMaxChannels  = 16;
const int    InputWidth_Bits   = 24;
const int    InputMuxCount     = 0;
const bool   InputIsUnipolar   = false;
const int    InputClockFactor  = 256;
const float  InputMaxRate      = 144.0e3f;
const float  InputMinRate      = 0.0;
const int    InputGangSize     = 1;
const float  InputReferenceClk = 100.0e6f;

const int    Output_MaxChannels  = 16;
const int    Output_Width_Bits   = 24;
const int    Output_MuxCount     = 0;
const bool   Output_IsUnipolar   = false;
const int    Output_ClockFactor  = 256;
const float  Output_MaxRate      = 192.0e3;
const float  Output_MinRate      = 0.0;
const int    Output_GangSize     = 1;
const float  Output_ReferenceClk = 100.0e6;

const float  HighSpeedThreshold  = 52734.0f;
const float  LowSpeedThreshold   = 10547.0f;
const double MaxAdjustment       = 0.125f;

//---------------------------------------------------------
//  IDROM Section Map
//---------------------------------------------------------
//  IDROM Section Map
//  ...First section: Input Coefficient section
//       (# channels x 2 values/ch + 1 calibration flag + 1 spare) x # ranges
const int romX3_SD16_InputCoeffOffset = 0;
const int romX3_SD16_InChannels = InputMaxChannels;
const int romX3_SD16_Ranges = 3;
const int romX3_SD16_RangeSize = romX3_SD16_InChannels * 2 + 1 + 1;
const int romX3_SD16_InputCoeffSize = romX3_SD16_RangeSize * romX3_SD16_Ranges;
const int romX3_SD16_InputCoeff_END = romX3_SD16_InputCoeffOffset + romX3_SD16_InputCoeffSize;
//  ...Second  section: Clock Parameters
//       (1 param + 1 calibrated word + 2 spare)
const int romX3_SD16_ClockParamOffset = romX3_SD16_InputCoeff_END;
const int romX3_SD16_ClockParamSize = 4;
const int romX3_SD16_ClockParam_END = romX3_SD16_ClockParamOffset + romX3_SD16_ClockParamSize;
//  ...Third  section: Output Coefficient Section Parameters
//       (1 param + 1 calibrated word + 2 spare)
const int romX3_SD16_OutputCoeffOffset  = romX3_SD16_ClockParam_END;
const int romX3_SD16_OutChannels        = Output_MaxChannels;
const int romX3_SD16_OutputCoeffSize    = romX3_SD16_OutChannels * 2 + 1 + 1;
const int romX3_SD16_OutputCoeff_END    = romX3_SD16_OutputCoeffOffset + romX3_SD16_OutputCoeffSize;
// 140 words used.
const int romX3_SD16_Reserved = 0x40;
const int romX3_SD16_EEProm_END = romX3_SD16_OutputCoeff_END + romX3_SD16_Reserved;

//===========================================================================
//  CLASS X3_SD16InfoIntf  -- Information about Inputs
//===========================================================================

class X3_SD16InfoIntf : public IoInfoIntf
{
public:
    X3_SD16InfoIntf()
        : FClockFactor(InputClockFactor),
               //    devices,         bits,            muxChannelCount,   unipolar
          FChannels(InputMaxChannels, InputWidth_Bits, InputMuxCount,     InputIsUnipolar)
        {
        FSpan.push_back(SpanInfo(10000, -10000));
        FSpan.push_back(SpanInfo(5000, -5000));
        FSpan.push_back(SpanInfo(1000, -1000));
        }
    virtual ~X3_SD16InfoIntf()  {  }


    virtual UnpairedChannelInfo & Channels() 		    {  return FChannels;  }
    virtual float               MaxRate() const			{  return InputMaxRate;  }
    virtual float               MinRate() const			{  return InputMinRate;  }
    virtual const SpanArray &   Span() const			{  return FSpan;  }
    virtual int                 GangSize() const		{  return InputGangSize;  }
    virtual bool                IsStackable() const		{  return false;  }
    virtual bool                IsInverted() const		{  return true;  }
    virtual bool                IsMuxed() const			{  return false;  }
    virtual bool                MuxDepth() const		{  return InputMuxCount;  }
    virtual int                 BufferSize() const		{  return 65536;  }
    virtual int                 ClockFactor() const		{  return FClockFactor;  }
    virtual float               ReferenceClock() const	{  return InputReferenceClk;  }
    virtual int               	SamplesPerFrame() const {  return 1;  }

    void                        SetClockFactor(unsigned int factor)
                                    {  FClockFactor = factor;  }
private:
    unsigned int                FClockFactor;
    UnpairedChannelInfo	        FChannels;
    SpanArray					FSpan;
};

//===========================================================================
//  CLASS X3_SD16OutputInfoIntf  -- Information about Outputs
//===========================================================================

class X3_SD16OutputInfoIntf : public IoInfoIntf
{
public:
    X3_SD16OutputInfoIntf()
        : FClockFactor(Output_ClockFactor),
               //    devices,                 bits,        muxChannelCount,      unipolar
          FChannels(Output_MaxChannels, Output_Width_Bits, Output_MuxCount,  Output_IsUnipolar)
        {  FSpan.push_back(SpanInfo(2000, -2000));  }
    virtual ~X3_SD16OutputInfoIntf()  {  }

    virtual UnpairedChannelInfo &	    Channels() 		        {  return FChannels;  }
    virtual float               		MaxRate() const			{  return Output_MaxRate;  }
    virtual float               		MinRate() const			{  return Output_MinRate;  }
    virtual const SpanArray &   		Span() const			{  return FSpan;  }
    virtual int                 		GangSize() const		{  return Output_GangSize;  }
    virtual bool                		IsStackable() const		{  return false;  }
    virtual bool                		IsInverted() const		{  return true;  }
    virtual bool                		IsMuxed() const			{  return false;  }
    virtual bool                		MuxDepth() const		{  return Output_MuxCount;  }
    virtual int                 		BufferSize() const		{  return 65536;  }
    virtual int                 		ClockFactor() const		{  return FClockFactor;  }
    virtual float               		ReferenceClock() const	{  return Output_ReferenceClk;  }
    virtual int               	        SamplesPerFrame() const {  return 1;  }

    void                                SetClockFactor(unsigned int factor)
                                            {  FClockFactor = factor;  }
private:
    unsigned int                FClockFactor;
    UnpairedChannelInfo	        FChannels;
    SpanArray					FSpan;
};

//==============================================================================
//  CLASS X3_SD16Clock -- X3_SD16 specific 9511 implementation
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class X3_SD16Clock
//---------------------------------------------------------------------------

X3_SD16Clock::X3_SD16Clock(X3_SD16DeviceMap * owner, int index,
    AddressingSpace &space, int ctl)
    : inherited(index, space, ctl), Owner(owner),
      Vcxo_I2C(Owner->VcxoIntf, I2CDevice::SubAddress, VcxoI2C_SubAddr),
      Vcxo(Vcxo_I2C, Si570::tcC, kVcxoPowerOnRate), FLowPower(false), FPDF(100.0e3)
{
    LoadVco(new IVcxoControlProxy(&Vcxo));
    UseAdvancedPBACalculation(true);
    Tuner.PfdRate(PhaseDetectorFrequency());
}

//---------------------------------------------------------------------------
//  constructor for class X3_SD16Clock
//---------------------------------------------------------------------------

X3_SD16Clock::X3_SD16Clock(X3_SD16DeviceMap * owner, double freq, double ref,
    bool enabled, int index, AddressingSpace & space, int ctl)
    : inherited(freq, ref, enabled, index, space, ctl), Owner(owner),
      Vcxo_I2C(Owner->VcxoIntf, I2CDevice::SubAddress, VcxoI2C_SubAddr),
      Vcxo(Vcxo_I2C, Si570::tcC, kVcxoPowerOnRate), FLowPower(false), FPDF(100.0e3)
{
    LoadVco(new IVcxoControlProxy(&Vcxo));
    UseAdvancedPBACalculation(true);
    Tuner.PfdRate(PhaseDetectorFrequency());
}

//---------------------------------------------------------------------------
//  X3_SD16Clock::ConfigureOutputs()  -- Tx specific output config
//---------------------------------------------------------------------------

void X3_SD16Clock::ConfigureOutputs(int outdiv_bits, int outdiv_bypass)
{
    PllConfigureOutputEvent e(*this, outdiv_bits, outdiv_bypass);
    OnPllConfigureOutput.Execute(e);

    // Identical duty cycle and phase for all outputs
    for (int i = 0; i < 8; ++i)
        {
        Port(0x48+i*2, outdiv_bits);	    // duty cycle
        Port(0x48+(i*2+1), outdiv_bypass);	// phase
        }

    Port(0x3C, 0x08);       // Clock 0 LVPECL on @810mV

    Port(0x3D, 0x03);       // ...powered off
    Port(0x3E, 0x03);       // ...powered off
    Port(0x3F, 0x03);       // ...powered off

    Port(0x40, 0x0a);       // Clocks 4..7 LVCMOS on @3.5 mA
    Port(0x41, 0x0a);
    Port(0x42, 0x0a);
    Port(0x43, 0x1a);

    PllConfigureOutputEvent e1(*this, outdiv_bits, outdiv_bypass);
    OnAfterPllConfigureOutput.Execute(e1);
}

//---------------------------------------------------------------------------
//  X3_SD16Clock::SetReferenceCalibrationFactor()  --
//---------------------------------------------------------------------------

void X3_SD16Clock::SetReferenceCalibrationFactor(double value)
{
    Owner->ClockParamRomSection.AsFloat(0, static_cast<float>(value));
    Owner->ClockParamRomSection.AsInt(1, static_cast<int>('Y'));    // set calibrated field
}

//---------------------------------------------------------------------------
//  X3_SD16Clock::GetReferenceCalibrationFactor()  --
//---------------------------------------------------------------------------

double X3_SD16Clock::GetReferenceCalibrationFactor() const
{
    //  Check Calibrated bit
    if (Owner->ClockParamRomSection.AsInt(1)=='Y')
        return static_cast<double>(Owner->ClockParamRomSection.AsFloat(0));
    else
        return 1.0;  // default value
}

//---------------------------------------------------------------------------
//  X3_SD16Clock::GetReferenceCalibrationFactor()  --
//---------------------------------------------------------------------------

X3_SD16Clock::SamplingMode X3_SD16Clock::SelectSamplingMode(double freq)
{

    //CLK input divider control: 1 = 37MHz (High-Speed mode)/otherwise 27MHz
    //                           0 = 13.5MHz (low-power)/5.4MHz (low-speed)
    //  MODE[1:0] Selects High-Speed, High-Resolution, Low-Power, or Low-Speed

    if (FLowPower)
        return (freq > LowSpeedThreshold) ? smLowPower : smLowSpeed;
    else
        return (freq > HighSpeedThreshold) ? smHighSpeed : smHiRes;
}

//---------------------------------------------------------------------------
//  X3_SD16Clock::SelectClockFactor()  --
//---------------------------------------------------------------------------

int X3_SD16Clock::SelectClockFactor(double freq)
{
    if (FLowPower)
        return (freq > LowSpeedThreshold) ? 512 : 2560;
    else
        return (freq > HighSpeedThreshold) ? 256 : 512;
}

//---------------------------------------------------------------------------
//  X3_SD16Clock::SetFrequency()  --
//---------------------------------------------------------------------------

void X3_SD16Clock::SetFrequency(double freq)
{
    SamplingMode mode = SelectSamplingMode(freq);
    int cf = SelectClockFactor(freq);

    ClockFactor(cf);

    X3_SD16InfoIntf & info = dynamic_cast<X3_SD16InfoIntf&>(Owner->Input().Info());
    info.SetClockFactor(cf);

    Owner->SetFSO(mode);

    Tuner.PfdRate(PhaseDetectorFrequency());
    inherited::SetFrequency(freq);

    Owner->ResetAnalog();
}

//===========================================================================
//  CLASS X3_SD16InputDevice  -- Module-specific input device features
//===========================================================================

//---------------------------------------------------------------------------
//  constructor
//  NOTE: Coefficient unity value adjusted below to preserve accuracy
//        within base-class intermediate calculations.  HW uses 1=0x10000
//---------------------------------------------------------------------------

X3_SD16InputDevice::X3_SD16InputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg)
    : inherited(owner, reg, new X3_SD16InfoIntf, 0x10000), // 'one' value for gain registers
      FLowPower(false)
{
    for (int i = 0; i < romX3_SD16_Ranges; ++i)
        {
        AppendRange(PmcIdromSection(owner->Rom(), PmcIdrom::waModule,
            i * romX3_SD16_RangeSize, romX3_SD16_RangeSize));
        }
}

//---------------------------------------------------------------------------
//  X3_SD16InputDevice::SetCalibrationOffset()
//---------------------------------------------------------------------------

void X3_SD16InputDevice::SetCalibrationOffset(size_t range, int ch, int value)
    {
    // Compensate for firmware, which left-shifts five bits
    if (range == GainControlReg[ch].Value())
        OffsetCoeffReg[ch].Value(value >> 5);
    }

//---------------------------------------------------------------------------
//  X3_SD16InputDevice::SetCalibrationGain()
//---------------------------------------------------------------------------

void X3_SD16InputDevice::SetCalibrationGain(size_t range, int ch, int value)
    {
    // Do nothing if inactive gain range specified
    if (range != GainControlReg[ch].Value())
        return;

//	// Recover original gain scale factor, as double
//	double x = static_cast<double>(value)/FOne;
//	x = std::max<double>(std::min<double>(x, 1.0f+MaxAdjustment), 1.0f-MaxAdjustment);
//
//	//  Reduce back to integer version
//	unsigned int gain = static_cast<unsigned int>(x * FOne);

    unsigned int gain = value;
    GainCoeffReg[ch].Value(gain);
    }

//---------------------------------------------------------------------------
//  X3IoDeviceRegisters X3_SD16InputDeviceRegisters --  Register address collection
//---------------------------------------------------------------------------

static X3IoDeviceRegisters X3_SD16InputDeviceRegisters =
    { mmI2c, 0,
      mmInControl, mmInGainCoef, mmInOffsetCoef, mmInGainMode,
      mmInChannelEnable, mmInAmpEnable, mmInTriggerControl,
      mmInDecimation, mmInPacketHeader, 0
    };


//===========================================================================
//  CLASS X3_SD16OutputDevice  -- X3_SD16 Output Device Definitions
//===========================================================================

//---------------------------------------------------------------------------
//  constructor for X3_SD16OutputDevice
//  NOTE: Coefficient unity value adjusted below to preserve accuracy
//        within base-class intermediate calculations.  HW uses 1=0x10000
//---------------------------------------------------------------------------

X3_SD16OutputDevice::X3_SD16OutputDevice(X3DeviceMap * owner,
                                       X3IoDeviceRegisters & reg)
    : inherited(owner,
                reg,
                new X3_SD16OutputInfoIntf,
                PmcIdromSection(owner->Rom(),
                                PmcIdrom::waModule,
                                romX3_SD16_OutputCoeffOffset,
                                romX3_SD16_OutputCoeffSize),
                0x10000)	// 'one' value for gain registers
{
}

//---------------------------------------------------------------------------
//  X3_SD16OutputDevice::SetCalibrationOffset()
//---------------------------------------------------------------------------

void X3_SD16OutputDevice::SetCalibrationOffset(int ch, int value)
    {
    // Compensate for firmware, which left-shifts five bits
    OffsetCoeffReg[ch].Value(value >> 5);
    }

//---------------------------------------------------------------------------
//  X3_SD16OutputDevice::SetCalibrationGain()
//---------------------------------------------------------------------------

void X3_SD16OutputDevice::SetCalibrationGain(int ch, int value)
    {
    // Recover original gain scale factor, as double
//	double x = static_cast<double>(value)/FOne;
//	x = std::max<double>(std::min<double>(x, 1.0f+MaxAdjustment), 1.0f-MaxAdjustment);
//
//	//  Reduce back to integer version
//	unsigned int gain = static_cast<unsigned int>(x * FOne);

    unsigned int gain = value;
    GainCoeffReg[ch].Value(gain);
    }

//===========================================================================

static X3IoDeviceRegisters X3_SD16OutputDeviceRegisters =
    { mmI2c, 0,
      mmOutControl, mmOutGainCoef, mmOutOffsetCoef, 0,
      mmOutChannelEnable, /*mmOutAmpEnable*/ 0, mmOutTriggerControl,
      mmOutDecimation, mmOutPacketHeader, 0
    };

//===========================================================================
//  CLASS X3_SD16DeviceMap  -- Register & low level support for SD module
//===========================================================================

//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

X3_SD16DeviceMap::X3_SD16DeviceMap(X3_SD16 * owner)
    : IUsesStreamDeviceEventCallbacks(owner),
      IUsesFrontPanelPort(LogicMemory, mmFrontPanelData, mmFrontPanelConfig),
      FClockSelect(csInternal),
      FInput(new X3_SD16InputDevice(this, X3_SD16InputDeviceRegisters)),
      FOutput(new X3_SD16OutputDevice(this, X3_SD16OutputDeviceRegisters)),
      Owner(owner),
      Clock(this, 0, LogicMemory, mmPll),
      ClockParamRomSection(Rom(), PmcIdrom::waModule,
                            romX3_SD16_ClockParamOffset, romX3_SD16_ClockParamSize)
{
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &X3_SD16DeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X3_SD16DeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X3_SD16DeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X3_SD16DeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X3_SD16DeviceMap::OnAfterStreamStopHandler );

    Rom().ModuleAreaWords(romX3_SD16_EEProm_END);

}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor
//---------------------------------------------------------------------------

X3_SD16DeviceMap::~X3_SD16DeviceMap()
{
    delete FInput;
    FInput = 0;

    delete FOutput;
    FOutput = 0;
}

//---------------------------------------------------------------------------
//  X3_SD16DeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X3_SD16DeviceMap::OpenHardware()
{
    ScopedTraceRef ccc(Debug, "X3_SD16DeviceMap::OpenHardware()");
    X3DeviceMap::OpenHardware();

    // Re-apply all Vcxo settings
    Debug.Log("At Vcxo Init()");
    Clock.Init();
    Debug.Log("After Vcxo Init()");

    // Configure A/Ds
    Input().PacketId(0x0);
    Input().PacketSize(0x4000);
    Input().Unframed();
    for (size_t range = 0; range < Input().Ranges(); ++range)
        {
        for (int ch = 0; ch < Input().Info().Channels().Channels(); ++ch)
            {
            Input().Gain(range, ch, 1.0);
            Input().Offset(range, ch, 0.0);
            }
        }

    // Configure D/As
    Output().PacketSize(0x4000);
    Output().PacketId(0x2);
    Output().Unframed();
    for (int ch = 0; ch < Output().Info().Channels().Channels(); ++ch)
        {
        FOutput->Gain(ch, 1.0);
        FOutput->Offset(ch, 0.0);
        }

    // hit software reset to avoid "noisy channel" error
    PokeLogic(0x60, 0xa80);   // assert software reset on device 0
    PokeLogic(0x60, 0xa00);   // deassert software reset on device 0

    PokeLogic(0x61, 0xa80);   // assert software reset on device 1
    PokeLogic(0x61, 0xa00);   // deassert software reset on device 1
}

//---------------------------------------------------------------------------
//  X3_SD16DeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X3_SD16DeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    // Family-common init
    X3DeviceMap::ConfigureHardware();

    // Reapply Hardware Settings for  X3 IO Devices
    FInput->ConfigureHardware();
    FOutput->ConfigureHardware();

    // Clear any FIFO overrun condition
    FInput->AnalogControlReg.FailedReset.Toggle();

    // Disable sync, for now.
    Sync(true);

    //
    //  Apply Clock Settings from saved state
    Clock.Reapply();

    //
    //  Apply Front config settings
    FrontPanelPortConfig(FrontPanelPortConfig());

    //
    //  Apply Gain/Offset Coefficients
    for (size_t i = 0; i < FInput->Ranges(); ++i)
        {
        if (FInput->Calibrated(i))
            FInput->ApplyRomCoefficients(i);
        else
            FInput->ApplyDefaultCoefficients(i);
        }

    if (FOutput->Calibrated())
        FOutput->ApplyRomCoefficients();
    else
        FOutput->ApplyDefaultCoefficients();

    //
    //  Apply input settings
    Input().PacketId(Input().PacketId());
    Input().PacketSize(Input().PacketSize());
    for (int ch = 0; ch < FInput->Info().Channels().Channels(); ++ch)
        {
        FInput->ChannelEnableReg.Bit(ch, FInput->Info().Channels().Enabled(ch));
        FInput->Range(ch, FInput->Range(ch));
        }
    //
    //  Apply output settings
    FOutput->PacketId(FOutput->PacketId());
    FOutput->PacketSize(FOutput->PacketSize());
    for (int ch = 0; ch < FOutput->Info().Channels().Channels(); ++ch)
        FOutput->ChannelEnableReg.Bit(ch, FOutput->Info().Channels().Enabled(ch));

    // Configure triggers
    FInput->TriggerReg.Apply();
    FOutput->TriggerReg.Apply();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  X3_SD16DeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X3_SD16DeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
	// hit software reset to avoid "noisy channel" error
	PokeLogic(0x60, 0xa80);   // assert software reset on device 0
	PokeLogic(0x60, 0xa00);   // deassert software reset on device 0

	PokeLogic(0x61, 0xa80);   // assert software reset on device 1
	PokeLogic(0x61, 0xa00);   // deassert software reset on device 1

    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStart.Execute(Event);
    //
    //  Start execution
    UserFpgaControlReg.InGo.Set();
    UserFpgaControlReg.OutGo.Set();
    UserFpgaControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X3_SD16DeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X3_SD16DeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);

    // Assert sync
    if (Master())
        Sync(false);
}

//---------------------------------------------------------------------------
//  X3_SD16DeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X3_SD16DeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3_SD16DeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X3_SD16DeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Disable User Logic Streaming
    UserFpgaControlReg.InGo.Clear();
    UserFpgaControlReg.OutGo.Clear();
    UserFpgaControlReg.Apply();

    Trigger(tInputOutput, false);

    // Deassert sync
    if (Master())
        Sync(true);

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3_SD16DeviceMap::Input() -- Reference to input object
//---------------------------------------------------------------------------

X3RangedIoDevice & X3_SD16DeviceMap::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  X3_SD16DeviceMap::Output() -- Reference to output object
//---------------------------------------------------------------------------

OutDevice & X3_SD16DeviceMap::Output()
{
    return *FOutput;
}

//---------------------------------------------------------------------------
//  X3_SD16DeviceMap::ConfigureFilters() -- Configure A/D filters and modes
//---------------------------------------------------------------------------

int X3_SD16DeviceMap::ConfigureFilters(float /*rate*/)
{
    int result = 1;
    return result;  // clock factor
}

//---------------------------------------------------------------------------
//  X3_SD16DeviceMap::SetFSO() -- Set operation mode bits
//---------------------------------------------------------------------------

void  X3_SD16DeviceMap::SetFSO(unsigned int value)
{
    FInput->AnalogControlReg.Fso.Value(value);
}

//---------------------------------------------------------------------------
//  X3_SD16DeviceMap::ResetAnalog() -- Reset Analog Chip
//---------------------------------------------------------------------------

void  X3_SD16DeviceMap::ResetAnalog()
{
    //  Reset analog
    FInput->AnalogControlReg.Reset.Set();
    FInput->AnalogControlReg.Reset.Clear();
    Sleep(1);   // delay 1 ms
}


} // namespace Innovative


